﻿/* Copyright (c) 2010 Lexalytics Inc.

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;

namespace LexaScopeNET.Zephyr
{
    /// <summary>
    /// Main class for the root of an asset result from Zephyr
    /// </summary>
    public class Result
    {
        /// <summary>Document identifier</summary>
        public string Identifier = "";
        /// <summary>Sentiment score for document</summary>
        public double Sentiment = 0.0;

        public List<Entity> AssetEntities = new List<Entity>();
        public List<Theme> AssetThemes = new List<Theme>();
        public List<Metadata> AssetMetadata = new List<Metadata>();
        public List<Sentiment> AssetSentiment = new List<Sentiment>();

        // Assignment constructor
        public Result(string strIdentifier, double dSentiment)
        {
            Identifier = strIdentifier;
            Sentiment = dSentiment;
        }
    }

    /// <summary>
    /// Class encapsulating entity result
    /// </summary>
    public class Entity
    {
        /// <summary>Entity text</summary>
        public string Text = "";

        /// <summary>Entity type</summary>
        public string Type = "";

        public string Label = "";

        public int Sentiment = 0;

        public double Magnitude = 0.0;

        public int Mentions = 0;

    }

    public class Theme
    {
        public string Text = "";
        public double Score = 0.0;
        public int Type = 0;
        public int Sentiment = 0;
        public double Magnitude = 0.0;
        public int Evidence = 1;
        public string Summary = "";
    }

    public class Metadata
    {
        public string Label = "";
        public string Value = "";
    }

    public class Sentiment
    {
        public double Score = 0.0;
        public List<SentimentPhrase> Phrases = new List<SentimentPhrase>();
        public Dictionary<string, double> ModelScores = new Dictionary<string, double>();
    }

    public class SentimentPhrase
    {
        public string Text = "";
        public double Score = 0.0;
        public int Type = 0;
        public int Inverted = 0;
        public string InvertingPhrase = "";
    }
}
