﻿/* Copyright (c) 2010 Lexalytics Inc.

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace Lexalytics4Excel.Dialogs
{
    /// <summary>
    /// About box for the Excel plug-in to provide version and support information
    /// </summary>
    public partial class About : Form
    {
        private string _Version = "<version>";
        private string _BuildDate = "<build date>";
        private string _Err = "";

        /// <summary>
        /// Dialog constructor
        /// </summary>
        public About()
        {
            InitializeComponent();
            // Retrieve version number and build date from assembly info
            try
            {
                Module[] appFiles = Assembly.GetExecutingAssembly().GetModules();
                foreach (Module appFile in appFiles)
                {
                    if (appFile.FullyQualifiedName.Contains("Lexalytics4Excel"))
                    {
                        _Version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
                        System.IO.FileInfo fileInfo = new System.IO.FileInfo(appFile.FullyQualifiedName);
                        _BuildDate = fileInfo.LastWriteTime.ToString();
                    }
                }
            }
            catch (Exception err)
            {
                _Err = err.Message;
            }


        }

        #region Event handlers
        private void About_Load(object sender, EventArgs e)
        {
            BuildVersionText.Text = _Version;
            BuildDateText.Text = _BuildDate;
        }

        private void SupportLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(String.Format("mailto:support@lexalytics.com?subject=Lexalytics Excel&body=Version: {0}, Build date: {1}", _Version, _BuildDate));
        }

        private void CloseDlgButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        #endregion // Event handlers
    }
}
