using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Runtime.Serialization;

namespace Lexalytics
{
#region MANAGED .NET LAYER

    public class SalienceStartup
    {
	    public string sLogPath;
	    public string sDataDirectory;
	    public string sUserDirectory;
    }

    public class SaliencePhrase
    {
        public string sText;
        public int nDocument;
        public int nSentence;
        public int nWord;
        public int nLength;
        public int nByte;
        public int nByteLength;
    }

    public class SalienceTheme
    {
        public string sTheme;
        public string sStemmedTheme;
        public string sNormalizedTheme;
        public int nThemeType;
        public float fScore;
        public float fSentiment;
        public int nEvidence;
        public bool bAbout;
        public string sSummary;
        public List<SaliencePhrase> ThemeMentions = new List<SaliencePhrase>();
    }

    public class SalienceSentiment
    {
        public float fScore;
        public List<SalienceSentimentPhrase> Phrases = new List<SalienceSentimentPhrase>();
        public List<SalienceModelSentiment> ModelSentiment = new List<SalienceModelSentiment>();
        
    }

    public class SalienceSentimentPhrase
    {
        public SaliencePhrase Phrase;
        public string sSource;
        public float fScore;
        public int nType;
        public int nNegated;
        public SaliencePhrase SupportingPhrase;
    }

    public class SalienceModelSentiment
    {
        public string sName;
        public int nBest;
        public float fPositive;
        public float fNegative;
        public float fMixed;
        public float fNeutral;
    }
    
    public class SalienceMention
    {
        public SaliencePhrase Phrase;
        public int nType;
        public float fScore;
    }
    
    public class SalienceEntity
    {
        public string sNormalizedForm;
        public string sType;
        public string sLabel;
        public float fSentimentScore;
        public int nEvidence;
        public int nConfident;
        public int nAbout;
        public string sSummary;
        public int nCount;
        public int nFirstPos;
        public List<SalienceMention> Mentions;
        public List<SalienceTheme> Themes;
        public List<SalienceSentimentPhrase> SentimentPhrases= new List<SalienceSentimentPhrase>();
    }

    public class SalienceCollectionEntity
    {
        public string sNormalizedForm;
        public string sType;
        public string sLabel;
        public int nCount;
        public int nPositiveCount;
        public int nNegativeCount;
        public int nNeutralCount;
        public List<SaliencePhrase> oMentions;
    }

    public class SalienceEntityParams
    {
        public int nRequiredThreshold = 55;
        public int nSummaryLength = 2;
        public int nTimeout = 5;
    }

    public class SalienceAttribute
    {
        public string sAttribute;
        public int nCount;
        public List<SaliencePhrase> Mentions;
    }

    public class SalienceFacet
    {
        public string sFacet;
        public int nCount;
        public int nPositiveCount;
        public int nNegativeCount;
        public int nNeutralCount;
        public List<SalienceAttribute> Attributes;
        public int nSubjectLength;
        public List<SaliencePhrase> Mentions;
    }

    public class SalienceToken
    {
        public string sToken;
        public int nCount;
    }

    public class SalienceWord
    {
        public string sToken;
	    public string sPOSTag;
        public string sStem;
	    public bool bInvert;
        public float fSentiment;
        public int nSentimentType;
        public string sEntityType;
    }

    public class SalienceChunk
    {
        public string sLabel;
	    public int nSentence;
        public List<SalienceWord> Tokens;     
    }

    public class SalienceSentence
    {
        public List<SalienceWord> Tokens;
        public List<SalienceChunk> Chunks;
        public int nLength;
        public int nChunkCount;
        public int nSubjective;
    }

    public class SalienceDocumentDetails
    {
        public string sText;
        public string sFingerprint;
        public int nWordCount;
        public int nSentenceCount;
        public int nObjectiveCount;
        public int nSubjectiveCount;
        public int nParsedCount;
        public List<SalienceToken> TermFrequency;
        public List<SalienceToken> BiGramFrequency;
        public List<SalienceToken> TriGramFrequency;
        public List<SalienceSentence> Sentences;
        public List<SalienceChunk> Chunks;
    }

    public class SalienceCollectionDetails
    {
        public int nSize;
    }

    public class SalienceRelationship
    {
        public float fScore;
        public string sLabel;
        public string sExtra;
        public List<SalienceEntity> Entities;
    }

    public class SalienceOpinion
    {
	    public SalienceEntity Speaker;
	    public SalienceEntity EntityTopic;
	    public SalienceTheme ThemeTopic;
        public string sQuotation;
        public float fSentiment;
        public bool bThemeOpinion;
     }

    public class SalienceTopic
    {
        public string sTopic;
        public int nHits;
        public float fScore;
        public List<string> vDocuments;
    }

    public class SalienceCollectionDocument
    {
        public string sIdentifier;
        public string sText;
    }

    public class SalienceCollection
    {
        public string sName;
        public int nSize;
        public List<SalienceCollectionDocument> Documents;
    }

#endregion // MANAGED .NET LAYER

#region UNMANAGED C LAYER (CONNECTION TO UNDERLYING SALIENCE C API)

    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public unsafe delegate int umSalienceCallback(void* pParam, int nStatus, IntPtr acMessage);

    [StructLayout(LayoutKind.Sequential)]
    unsafe internal struct umSalienceStartup
    {
        [MarshalAsAttribute(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst = 1024)]
        public string acError;
        public int nStartupLog;
        public IntPtr acLogPath;
        [MarshalAsAttribute(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst = 260)]
        public string acDataDirectory;
        [MarshalAsAttribute(System.Runtime.InteropServices.UnmanagedType.ByValTStr, SizeConst = 260)]
        public string acUserDirectory;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal struct umSalienceOption
    {
        public int nOption;         //The option being set
        public IntPtr acValue;      //Use this member if the set option requires a string
        public int nValue;          //!< Use this member if the set option requires an integer
        public float fValue;        //!< Use this member if the set option requires a float
    }


    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSaliencePhrase
    {
        public IntPtr acText;
        public int nDocument;
        public int nSentence;
        public int nWord;
        public int nLength;
        public int nByte;
        public int nByteLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSaliencePhraseList
    {
        public umSaliencePhrase* pPhrases;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSuggestedTheme
    {
        public IntPtr acTheme;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceTheme
    {
        public IntPtr acTheme;
        public IntPtr acStemmedTheme;
        public IntPtr acNormalizedTheme;
        public int nThemeType;
        public float fScore;
        public float fSentiment;
        public int nEvidence;
        public int nAbout;
        public IntPtr acSummary;
        public umSaliencePhraseList oMentions;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceThemeList
    {
        public umSalienceTheme* pThemes;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceAttribute
    {
        public IntPtr acAttribute;
        public int nCount;
        public umSaliencePhraseList oMentions;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceFacet
    {
        public IntPtr acFacet;
        public IntPtr acSubFacetList;
        public int nCount;
        public int nPositiveCount;
        public int nNegativeCount;
        public int nNeutralCount;
        public umSalienceAttribute* pAttributes;
        public int nSubjectLength;
        public umSaliencePhraseList oMentions;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceFacetList
    {
        public umSalienceFacet* pFacetList;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSentimentPhrase
    {
        public umSaliencePhrase oPhrase;
        public float fScore;
        public int nType;
        public IntPtr acSource;
        public int nNegated;
        public umSaliencePhrase oSupportingPhrase;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSentimentPhraseList
    {
        public umSalienceSentimentPhrase* pPhrases;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSentimentModel
    {
        public int nBest;
        public float fPositive;
        public float fNegative;
        public float fMixed;
        public float fNeutral;
        public IntPtr acModelName;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSentimentResult
    {
        public float fScore;
        public umSalienceSentimentPhraseList oPhrases;
        public umSalienceSentimentModel* pModel;
        public int nModelCount;

    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceMention
    {
        public umSaliencePhrase oPhrase;
        public int nType;
        public float fScore;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceMentionList
    {
        public umSalienceMention* pMentions;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceEntity
    {
        public IntPtr acNormalizedForm;
        public IntPtr acType;
        public IntPtr acLabel;
        public float fSentimentScore;
        public int nEvidence;
        public int nConfident;
        public int nAbout;
        public IntPtr acSummary;
        public umSalienceMentionList oMentions;
        public umSalienceThemeList oThemes;
        public umSalienceSentimentPhraseList oSentimentPhrases;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceCollectionEntity
    {
        public IntPtr acNormalizedForm;
        public IntPtr acType;
        public IntPtr acLabel;
        public int nCount;
        public int nPositiveCount;
        public int nNegativeCount;
        public int nNeutralCount;
        public umSaliencePhraseList oMentions;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceEntityList
    {
        public umSalienceEntity* pEntities;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceCollectionEntityList
    {
        public umSalienceCollectionEntity* pCollectionEntities;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceRelation
    {
        public umSalienceEntityList oEntities;
        public IntPtr acType;
        public float fConfidence;
        public IntPtr acExtra;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceRelationList
    {
        public umSalienceRelation* pRelations;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceOpinion
    {
        public umSalienceEntity oSpeaker;
        public umSalienceEntity oEntityTopic;
        public umSalienceTheme oThemeTopic;
        public IntPtr acQuotation;
        public float fSentiment;
        public int nHasTheme;
    };

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceOpinionList
    {
        public umSalienceOpinion* pOpinions;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceToken
    {
        public IntPtr acToken;
        public int nCount;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceTokenList
    {
        public umSalienceToken* pTokens;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceWord
    {
        public IntPtr acToken;
        public IntPtr acPOSTag;
        public IntPtr acStem;
        public int nInvert;
        public float fSentiment;
        public int nSentimentType;
        public IntPtr acEntityType;
        public int nId;
        public int nSecondaryId;
        public int nPostFixed;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceChunk
    {
        public umSalienceWord* pTokens;
        public int nLength;
        public IntPtr acLabel;
        public int nSentence;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceSentence
    {
        public umSalienceWord* pTokens;
        public umSalienceChunk* pChunks;
        public int nLength;
        public int nChunkCount;
        public int nSubjective;
        public int nPolar;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceDocument
    {
        public int nSentenceCount;
        public umSalienceSentence* pSentences;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceDocumentDetails
    {
        public int nWordCount;                      //!< The number of tokens in the document.
        public int nSentenceCount;                  //!< The number of sentences in the document.
        public int nObjectiveCount;                 //!< The number of objective sentences in the document.
        public int nSubjectiveCount;                //!< The number of subjective sentences in the document.
        public int nParsedCount;                    //!< The number of sentences that grammatically parsed.
        public umSalienceTokenList oTermFrequency;
        public umSalienceTokenList oBiGrams;
        public umSalienceTokenList oTriGrams;
        public umSalienceSentence* pSentences;      //!< Pointer to a list of the individual sentences.
        public IntPtr acInternalRepresentation;
        public IntPtr acFingerprint;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceCollectionDetails
    {
        public int nSize;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceTopic
    {
        public IntPtr acTopic;
        public int nHits;
        public float fScore;
        public IntPtr acAdditional;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceTopicList
    {
        public umSalienceTopic* pTopics;
        public int nLength;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceCollectionDocument
    {
	    public IntPtr acIndentifier;
        public int nIsText;
        public int nSplitByLine;
	    public IntPtr acText;
    }

    [StructLayout(LayoutKind.Sequential)]
    internal unsafe struct umSalienceCollection
    {
        public IntPtr acName;
    	public int nSize;
	    public umSalienceCollectionDocument* pDocuments;
    }

#endregion // UNMANAGED C LAYER (CONNECTION TO UNDERLYING SALIENCE C API)
}
