using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Runtime.Serialization;

namespace Lexalytics
{
    public class SalienceFive : SalienceFiveEngine, IDisposable
    {

#region MANAGED .NET LAYER
    #region Document-specific options
        private bool m_UseSemantics;
        /// <summary>
        /// Determines whether or not to utilize the concept matrix during document operations
        /// </summary>
        public bool UseSemantics
        {
            get { return m_UseSemantics; }
            set { if (SetSalienceOption(1006, value) == 0) m_UseSemantics = value; }
        }
    #endregion // Document-specific options

    #region Constructors

        /// <summary>
        /// Standard constructor, requires path to license file and path to data directory
        /// </summary>
        /// <param name="sLicensePath">Path to a valid Lexalytics license file</param>
        /// <param name="sDataPath">Path to data directory to use for this session</param>
        /// <exception cref="SalienceException">
        /// Thrown if license file cannot be loaded or error in loading data directory (AccessViolationException), or unable to load underlying Salience Engine (DllNotFoundException)
        /// </exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveConstructor"/>
        public SalienceFive(string sLicensePath, string sDataPath) : base(sLicensePath, sDataPath) {}

        /// <summary>
        /// Standard constructor, requires path to license file and path to data directory
        /// </summary>
        /// <param name="sLicensePath">Path to a valid Lexalytics license file</param>
        /// <param name="sDataPath">Path to data directory to use for this session</param>
        /// <param name="sUserDir">Path to a user directory of customizations to use for this session</param>
        /// <exception cref="SalienceException">
        /// Thrown if license file cannot be loaded or error in loading data directory (AccessViolationException), or unable to load underlying Salience Engine (DllNotFoundException)
        /// </exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveConstructor"/>
        public SalienceFive(string sLicensePath, string sDataPath, string sUserDir) : base(sLicensePath, sDataPath, sUserDir) { }

        /// <summary>
        /// Standard constructor, requires path to license file and path to data directory
        /// </summary>
        /// <param name="sLicensePath">Path to a valid Lexalytics license file</param>
        /// <param name="sDataPath">Path to data directory to use for this session</param>
        /// <param name="sUserDir">Path to a user directory of customizations to use for this session</param>
        /// <param name="sLogPath">Path to a log file to write session in itialization information to</param>
        /// <exception cref="SalienceException">
        /// Thrown if license file cannot be loaded or error in loading data directory (AccessViolationException), or unable to load underlying Salience Engine (DllNotFoundException)
        /// </exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveConstructor"/>
        public SalienceFive(string sLicensePath, string sDataPath, string sUserDir, string sLogPath) : base(sLicensePath, sDataPath, sUserDir, sLogPath) { }

        /// <summary>
        /// Standard constructor, requires path to license file and path to data directory
        /// </summary>
        /// <param name="sLicensePath">Path to a valid Lexalytics license file</param>
        /// <param name="oStartup">Startup object giving other session startup parameters</param>
        /// <exception cref="SalienceException">
        /// Thrown if license file cannot be loaded or error in loading data directory (AccessViolationException), or unable to load underlying Salience Engine (DllNotFoundException)
        /// </exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveConstructor"/>
        /// <seealso cref="Insert URL for lxaLoadLicense and lxaOpenSalienceSession documentation"/>
        public SalienceFive(string sLicensePath, SalienceStartup oStartup) : base(sLicensePath, oStartup) {  }

    #endregion // Constructors

    #region Object disposal
        new public void Dispose()
        {
            base.Dispose();
        } 
    #endregion 

    #region Utility methods
        /// <summary>
        /// Wrapper method to return the Salience Engine version
        /// </summary>
        /// <returns>String with Salience Engine version information</returns>
        /// <remarks>Wrapper around C API method lxaGetSalienceVersion</remarks>
        /// <exception cref="SalienceException">Thrown if version information is not available</exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetSalienceVersion"/>
        new public string GetSalienceVersion()
        {
            return base.GetSalienceVersion();
        }

        /// <summary>
        /// Static method for retrieving the default Salience location without needing to instantiate an object
        /// </summary>
        /// <returns>String path to Salience installation</returns>
        new public static string GetDefaultSalienceLocation()
        {
            return SalienceFiveEngine.GetDefaultSalienceLocation();
        }

        /// <summary>
        /// Wrapper method to return the output of Salience Engine operating environment
        /// </summary>
        /// <returns>String with Salience Engine environment information</returns>
        /// <remarks>Wrapper around C API method lxaDumpEnvironment</remarks>
        /// <exception cref="SalienceException">Thrown if version information is not available</exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveDumpEnvironment"/>
        new public string DumpEnvironment()
        {
            return base.DumpEnvironment();
        }

        /// <summary>
        /// Returns an integer error code indicating last warning encountered by Salience Engine, generally in the process of text preparation
        /// </summary>
        /// <returns>Integer return code</returns>
        /// <remarks>Wrapper around C API method lxaGetLastWarnings</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetLastWarnings"/>
        new public int GetLastWarnings()
        {
            return base.GetLastWarnings();
        }

        /// <summary>
        /// Wrapper method that sets the underlying SentimentDictionary option with an additional reset flag parameter. 
        /// </summary>
        /// <param name="sDictionary">Path to a sentiment hand-scored dictionary (HSD).</param>
        /// <param name="bReset">Flag to clear all previously loaded HSD files</param>
        /// <exception cref="SalienceException">Thrown if setting the option fails.</exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveAddSentimentDictionary"/>
        new public int AddSentimentDictionary(string sDictionary, bool bReset)
        {
            return base.AddSentimentDictionary(sDictionary, bReset);
        }


    #endregion // Utility methods

    #region Text Preparation
        /// <summary>
        /// Wrapper method to submit text to Salience Engine for processing
        /// </summary>
        /// <param name="sText">The text to be processed</param>
        /// <returns>Integer error return code</returns>
        /// <remarks>Wrapper around C API method lxaPrepareText</remarks>
        /// <exception cref="SalienceException">Thrown if text cannot be parsed</exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFivePrepareText"/>
        new public int PrepareText(string sText)
        {
            return base.PrepareText(sText);
        }

        /// <summary>
        /// Wrapper method to submit contents of a file to Salience Engine for processing
        /// </summary>
        /// <param name="sFile">Full path to a readable text file</param>
        /// <returns>Integer error return code</returns>
        /// <remarks>Wrapper around C API method lxaPrepareTextFromFile</remarks>
        /// <exception cref="SalienceException">Thrown if file cannot be found or text cannot be parsed</exception>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFivePrepareTextFromFile"/>
        new public int PrepareTextFromFile(string sFile)
        {
            return base.PrepareTextFromFile(sFile);
        }

    #endregion // Text Preparation

    #region Details functions
        /// <summary>
        /// Wrapper method to return details about current document
        /// </summary>
        /// <returns>SalienceDocumentDetails object</returns>
        /// <remarks>Wrapper around C API method lxaGetDocumentDetails</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetDocumentDetails"/>
        new public SalienceDocumentDetails GetDocumentDetails()
        {
            return base.GetDocumentDetails();
        }

    #endregion // Details functions

    #region Document methods

        /// <summary>
        /// Returns a summary for the document
        /// </summary>
        /// <param name="nLength">The length in sentences that the summary should be.</param>
        /// <returns>A text string containing a summary of the document up to nLength sentences.</returns>
        /// <remarks>Wrapper around C API method lxaGetSummary</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetSummary"/>
        new public string GetSummary(int nLength)
        {
            return base.GetSummary(nLength);
        }

        /// <summary>
        /// Returns the sentiment analysis for the document
        /// </summary>
        /// <param name="bUseChains">A boolean indicating whether or not to use lexical chains for sentiment phrase analysis</param>
        /// <returns>A SalienceSentiment object containing members with sentiment results</returns>
        /// <remarks>Wrapper around C API method lxaGetSentiment</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetDocumentSentiment"/>
        new public SalienceSentiment GetDocumentSentiment(bool bUseChains)
        {
            return base.GetDocumentSentiment(bUseChains);
        }

        /// <summary>
        /// Returns a list of themes extracted from the document
        /// </summary>
        /// <returns>A List of SalienceTheme objects, each containing information about a theme extracted from the document</returns>
        /// <remarks>Wrapper around C API method lxaGetThemes</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetDocumentThemes"/>
        new public List<SalienceTheme> GetDocumentThemes()
        {
            return base.GetDocumentThemes();
        }

        /// <summary>
        /// Returns a list of topics extracted from the document via tag query definitions.
        /// This is equivalent to the SE4.4 method GetDocumentTags.
        /// </summary>
        /// <returns>A List of SalienceTopic objects, each containing information about a topic identified for the document</returns>
        /// <remarks>Wrapper around C API method lxaGetQueryDefinedTopics</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetQueryDefinedTopics"/>
        new public List<SalienceTopic> GetQueryDefinedTopics()
        {
            return base.GetQueryDefinedTopics();
        }

        /// <summary>
        /// Returns a list of topics extracted from the document via the concept matrix.
        /// </summary>
        /// <returns>A List of SalienceTopic objects, each containing information about a topic identified for the document</returns>
        /// <remarks>Wrapper around C API method lxaGetConceptDefinedTopics</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetConceptDefinedTopics"/>
        new public List<SalienceTopic> GetConceptDefinedTopics()
        {
            return base.GetConceptDefinedTopics();
        }


        /// <summary>
        /// Returns a string listing the top contributing words and phrases from the current document for each Concept Topic Match.
        /// </summary>
        /// <returns>A text string with concept topic results, and contributing words/phrases.</returns>
        /// <remarks>Wrapper around C API method lxaExplainConceptMatches</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveExplainConceptMatches"/>
        new public string ExplainConceptMatches()
        {
            return base.ExplainConceptMatches();
        }

    #endregion // Document methods

    #region Entity methods

        /// <summary>
        /// Returns a list of named entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntities.
        /// </summary>
        /// <returns>A List of SalienceEntity objects, each containing information about a named entity extracted from the document.</returns>
        /// <remarks>Wrapper around C API method lxaGetNamedEntities</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetNamedEntities"/>
        new public List<SalienceEntity> GetNamedEntities()
        {
            return base.GetNamedEntities();
        }

        /// <summary>
        /// Returns a list of named entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntities.
        /// </summary>
        /// <param name="Params">A parameter structure of options for entity extraction</param>
        /// <returns>A List of SalienceEntity objects, each containing information about a named entity extracted from the document.</returns>
        /// <remarks>Wrapper around C API method lxaGetNamedEntities</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetNamedEntities"/>
        new public List<SalienceEntity> GetNamedEntities(SalienceEntityParams Params)
        {
            return base.GetNamedEntities(Params);
        }

        /// <summary>
        /// Returns a list of named entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetSimpleEntities and GetQueryDefinedEntities.
        /// </summary>
        /// <returns>A List of SalienceEntity objects, each containing information about a user-defined entity extracted from the document.</returns>
        /// <remarks>Wrapper around C API method lxaGetUserDefinedEntities</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetUserDefinedEntities"/>
        new public List<SalienceEntity> GetUserDefinedEntities()
        {
            return base.GetUserDefinedEntities();
        }

        /// <summary>
        /// Returns a list of relationships for named entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntityRelationships.
        /// </summary>
        /// <returns>A List of SalienceRelationship objects, each containing information about a relationship between multiple named entities</returns>
        /// <remarks>Wrapper around C API method lxaGetNamedRelationships</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetNamedEntityRelationships"/>
        new public List<SalienceRelationship> GetNamedEntityRelationships(int nThreshold)
        {
            return base.GetNamedEntityRelationships(nThreshold);
        }

        /// <summary>
        /// Returns a list of opinions for named entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntityOpinions.
        /// </summary>
        /// <returns>A List of SalienceOpinion objects, each containing information about an opinion related to a named entity</returns>
        /// <remarks>Wrapper around C API method lxaGetNamedEntityOpinions</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetNamedEntityOpinions"/>
        new public List<SalienceOpinion> GetNamedEntityOpinions(int nThreshold)
        {
            return base.GetNamedEntityOpinions(nThreshold);
        }

        /// <summary>
        /// Returns a list of relationships for user-defined entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntityRelationships for named entities, but for simple and query-defined entities.
        /// </summary>
        /// <returns>A List of SalienceRelationship objects, each containing information about a relationship between multiple user-defined entities</returns>
        /// <remarks>Wrapper around C API method lxaGetUserEntityRelationships</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetUserEntityRelationships"/>
        new public List<SalienceRelationship> GetUserEntityRelationships()
        {
            return base.GetUserEntityRelationships();
        }

        /// <summary>
        /// Returns a list of opinions for user-defined entities extracted from the document.
        /// This is equivalent to the SE4.4 method GetEntityOpinions for named entities, but for simple and query-defined entities.
        /// </summary>
        /// <returns>A List of SalienceOpinion objects, each containing information about an opinion related to a user-defined entity</returns>
        /// <remarks>Wrapper around C API method lxaGetUserEntityOpinions</remarks>
        /// <see cref="http://dev.lexalytics.com/wiki-se5/pmwiki.php?n=SE5API.SalienceFiveGetUserEntityOpinions"/>
        new public List<SalienceOpinion> GetUserEntityOpinions()
        {
            return base.GetUserEntityOpinions();
        }

    #endregion // Entity methods

    #region Markup functions
        new public string GetEntityTaggedText()
        {
            return base.GetEntityTaggedText();
        }

        new public string GetUserEntityTaggedText()
        {
            return base.GetUserEntityTaggedText();
        }

        new public string GetPOSTaggedText()
        {
            return base.GetPOSTaggedText();
        }

        new public string GetSentimentTaggedText()
        {
            return base.GetSentimentTaggedText();
        }

        new public string GetNamedOpinionTaggedText()
        {
            return base.GetNamedOpinionTaggedText();
        }

        new public string GetUserOpinionTaggedText()
        {
            return base.GetUserOpinionTaggedText();
        }

    #endregion // Markup functions

    #region Deprecated methods
        /// <summary>
        /// DEPRECATED: This method has been replaced with the property SalienceFive.EntityList. 
        /// The method has been retained for backward compatibility, it sets the appropriate property. It may be removed in later releases.
        /// </summary>
        /// <remarks>DEPRECATED METHOD</remarks>
        /// <param name="sFile">Path to a CDL file containing user-defined entities to extract from content.</param>
        /// <exception cref="SalienceException">Thrown if setting the option fails.</exception>
        public void SetSimpleEntitiesFile(string sFile)
        {
            base.EntityList = sFile;
        }

        /// <summary>
        /// DEPRECATED: This method has been replaced with the property SalienceFive.UserDirectory. 
        /// The method has been retained for backward compatibility, it sets the appropriate property. It may be removed in later releases.
        /// </summary>
        /// <remarks>DEPRECATED METHOD</remarks>
        /// <param name="sFile">Path to a user data directory containing customized datafiles.</param>
        /// <exception cref="SalienceException">Thrown if setting the option fails.</exception>
        public void SetUserDirectory(string sDirectory)
        {
            base.UserDirectory = sDirectory;
        }

        /// <summary>
        /// DEPRECATED: This method has been replaced with the property SalienceFive.TopicList. 
        /// The method has been retained for backward compatibility, it sets the appropriate property. It may be removed in later releases.
        /// </summary>
        /// <remarks>DEPRECATED METHOD</remarks>
        /// <param name="sFile">Path to a file of topic queries.</param>
        /// <exception cref="SalienceException">Thrown if setting the option fails.</exception>
        public void SetTagDefinitions(string sDefinitions)
        {
            base.TopicList = sDefinitions;
        }

    #endregion // Deprecated methods

#endregion // MANAGED .NET LAYER
    }
}
